<?php
/**
 * Description: Takes the IP of the visitor. Returns an array of information about IP.
 *
 * PHP version 8.0.1
 *
 * @category webrtc2-ip-info.php
 * @package  WP-WebRTC2
 * @author   Oleg Klenitsky <klenitskiy.oleg@mail.ru>
 * @version  1.7.2
 * @license  GPLv2 or later
 * @filesource
 */

if ( ! defined( "ABSPATH" ) ) {
	exit();
}

/**
 * Select name of WHO-IS provider.
 *
 * @param  string $user_ip User ip.
 * @param  string $provider_who_is Name of who is provider.
 * @return array.
 */
function webrtc2_who_is( $user_ip, $provider_who_is ) {
	$check_ip = filter_var($user_ip, FILTER_VALIDATE_IP);
	if (!$check_ip){
		$user_ip  = gethostbyname($user_ip);
		$check_ip = filter_var($user_ip, FILTER_VALIDATE_IP);
		if (!$check_ip){
			return array();
		}
	}
	switch ( $provider_who_is ) {
		case "IP-API":
			return webrtc2_ip_api( $user_ip );
		case "IP-Info":
			return webrtc2_ip_info( $user_ip );
		case "SxGeo":
			return webrtc2_sx_geo( $user_ip );
		case "none":
			return array();
	}
}
/**
 * Retrieve ip information from the provider IP_API.
 *
 * @param  string $user_ip User ip.
 * @return array In format json.
 */
function webrtc2_ip_api( $user_ip ) {
	WP_Filesystem();
	global $wp_filesystem;

	$str_info     = array();
	// Receiving data from API  JSON - format.
	$country_info = json_decode( $wp_filesystem->get_contents( "http://ip-api.com/json/" . $user_ip ), true );

	// get user country code.
	$country_code = isset( $country_info["countryCode"] ) ? sanitize_text_field( $country_info["countryCode"] ) : "";

	if ($country_code == "") return array();

	// get user country name.
	$country_name = ( "" !== $country_code ) ? webrtc2_country_name( $country_code ) : "";
	// get user country region.
	$region = isset( $country_info["regionName"] ) ? sanitize_text_field( $country_info["regionName"] ) : "";
	// get user country city.
	$city = isset( $country_info["city"] ) ? sanitize_text_field( $country_info["city"] ) : "";
	// info.
	$str_info["country"]  = $country_code .
							"<br>Country: " . $country_name .
							"<br>Region: " . $region .
							"<br>City: " . $city;
	$provider             = isset( $country_info["as"] ) ? $country_info["as"] : "undefined";
	$str_info["provider"] = $provider;
	// timezone.
	$timezone             = isset( $country_info["timezone"] ) ? $country_info["timezone"] : "undefined";
	$str_info["timezone"] = $timezone;
	// IP address.
	$str_info["ip_address"] = $user_ip;
	// coordinates.
	$lat = isset( $country_info["lat"] ) ? sanitize_text_field( $country_info["lat"] ) : "";
	$lon = isset( $country_info["lon"] ) ? sanitize_text_field( $country_info["lon"] ) : "";

	$position = array("latitude"=>$lat, "longitude"=>$lon);
	$str_info["geo_ip"] = json_encode( $position );

	return $str_info;
}
/**
 * Retrieve ip information from the provider IP_info.
 *
 * @param  string $user_ip User ip.
 * @return array In format json.
 */
function webrtc2_ip_info( $user_ip ) {
	WP_Filesystem();
	global $wp_filesystem;

	$str_info     = array();
	// Receiving data from API  JSON - format.
	$country_info = json_decode( $wp_filesystem->get_contents( "http://ipinfo.io/" . $user_ip . "/json" ) );

	// get user country code.
	$country_code = isset( $country_info->country ) ? sanitize_text_field( $country_info->country ) : "";

	if ($country_code == "") return array();

	// get user country name.
	$country_name = ( "" !== $country_code ) ? webrtc2_country_name( $country_code ) : "";
	// get user country region.
	$region = isset( $country_info->region ) ? sanitize_text_field( $country_info->region ) : "";
	// get user country city.
	$city = isset( $country_info->city ) ? sanitize_text_field( $country_info->city ) : "";
	// info.
	$str_info["country"]  = $country_code .
							"<br>Country: " . $country_name .
							"<br>Region: " . $region .
							"<br>City: " . $city;
	$provider             = isset( $country_info->org ) ? $country_info->org : "undefined";
	$str_info["provider"] = $provider;
	// timezone.
	$timezone             = isset( $country_info->timezone ) ? $country_info->timezone : "undefined";
	$str_info["timezone"] = $timezone;
	// IP address.
	$str_info["ip_address"] = $user_ip;
	// coordinates.
	$loc = isset( $country_info->loc ) ? explode( ",", $country_info->loc ) : array();
	$lat = isset( $loc[0] ) ? sanitize_text_field( $loc[0] ) : "";
	$lon = isset( $loc[1] ) ? sanitize_text_field( $loc[1] ) : "";

	$position = array("latitude"=>$lat, "longitude"=>$lon);
	$str_info["geo_ip"] = json_encode( $position );

	return $str_info;
}
/**
 * Retrieve ip information from the provider SxGeo.
 *
 * @param  string $user_ip User ip.
 * @return array In format json.
 */
function webrtc2_sx_geo( $user_ip ) {
	WP_Filesystem();
	global $wp_filesystem;

	$str_info = array();
	// Receiving data from API  JSON - format.
	$geo_api_request = "http://api.sypexgeo.net/json/" . $user_ip;
	$geo_info_json   = $wp_filesystem->get_contents( $geo_api_request );
	// Receive associative array with result of request to API.
	$country_info = json_decode( $geo_info_json, true );

	// get user country code.
	$country_code = isset( $country_info["country"]["iso"] ) ? sanitize_text_field( $country_info["country"]["iso"] ) : "";

	if ($country_code == "") return array();

	// get user country name.
	$country_name = ( "" !== $country_code ) ? webrtc2_country_name( $country_code ) : "";
	// get user country region.
	$region = isset( $country_info["region"]["name_en"] ) ? sanitize_text_field( $country_info["region"]["name_en"] ) : "";
	// get user country city.
	$city = isset( $country_info["city"]["name_en"] ) ? sanitize_text_field( $country_info["city"]["name_en"] ) : "";
	// info.
	$str_info["country"]  = $country_code .
							"<br>Country: " . $country_name .
							"<br>Region: " . $region .
							"<br>City: " . $city;
	$provider             = "undefined";
	$str_info["provider"] = $provider;
	// timezone.
	$timezone = isset( $country_info["country"]["timezone"] ) ? $country_info["country"]["timezone"] : "undefined";
	$str_info["timezone"] = $timezone;
	// IP address.
	$str_info["ip_address"] = $user_ip;
	// coordinates.
	$lat = isset( $country_info["city"]["lat"] ) ? sanitize_text_field( $country_info["city"]["lat"] ) : "";
	$lon = isset( $country_info["city"]["lon"] ) ? sanitize_text_field( $country_info["city"]["lon"] ) : "";

	$position = array("latitude"=>$lat, "longitude"=>$lon);
	$str_info["geo_ip"] = json_encode( $position );

	return $str_info;
}
/**
 * Retrieve information of country of visitor.
 *
 * @param  string $country_code Country code.
 * @return string User country name.
 */
function webrtc2_country_name( $country_code ) {
	global $wpdb;

	$results = $wpdb->get_var(
		$wpdb->prepare(
			"
      SELECT `name`
      FROM {$wpdb->prefix}webrtc2_countries
      WHERE `code` = %s
      ",
			$country_code
		)
	);

	return $results;
}
